!
! nuclstatave --- average of data computed for an isotopic mixture
! Christian Buth
! 23 Aug 2007
!
! Version 1.0.0: 23 Aug 2007 (Compute nuclear statistics average of
!       data sets for diatomic molecules)
!


      program nuclstatave
      use fortrantl
      implicit none

      integer, parameter :: uhomoOne = 100, uhomoTwo = 101, uinhomo = 102

      real(dblprec) probOne, probTwo, enhomoOne, dathomoOne, enhomoTwo,  &
     &          dathomoTwo, eninhomo, datinhomo, avdata

      character(1024) homoOne, homoTwo, inhomo

      namelist/NuclStat/ homoOne, homoTwo, inhomo, probOne, probTwo

      ! Initialization
      write (*,*) 'nuclstatave V1.0.0, average of data computed for an isotopic mixture'
      write (*,*) '(c) 2007 Christian Buth'
      write (*,*)

      read(*,nml=NuclStat)

      open(unit=uhomoOne, file=homoOne, status='old', action='read',  &
     &          form='formatted', access='sequential')
      open(unit=uhomoTwo, file=homoTwo, status='old', action='read',  &
     &          form='formatted', access='sequential')
      open(unit=uinhomo,  file=inhomo, status='old', action='read',  &
     &          form='formatted', access='sequential')

      do
        read(uhomoOne, end=100, fmt=*) enhomoOne, dathomoOne
        read(uhomoTwo, end=100, fmt=*) enhomoTwo, dathomoTwo
        read(uinhomo,  end=100, fmt=*) eninhomo,  datinhomo
        if (enhomoOne /= enhomoTwo .or. enhomoOne /= eninhomo) then
          stop 'nuclstatave: energies must be the same throughout the files!'
        end if
        avdata = probOne**2 * dathomoOne + probTwo**2 * dathomoTwo  &
     &          + 2.0D0 * probOne * probTwo * datinhomo
        write(103,*) eninhomo, avdata
      end do

 100  continue

      close(uhomoOne)
      close(uhomoTwo)
      close(uinhomo)

      write (*,*) 'Statistical average finished!'
      end program nuclstatave
