!
! matrixvector --- matrix times block-vector for the matrix file
! Christian Buth
! 18 Sep 2006
!
!
! zmatblksparse --- matrix times vector product is the matrix
!     times vector routine which computes the product between
!     the np vectors pold and the matrix and stores the
!     np resulting vectors in pnew
!
!     external zmatblk(ndim, np, pold, pnew)
!


      subroutine matrixvector(ndim, np, pold, pnew)
      use sparse
      implicit none

      integer, intent(in) :: ndim, np
      complex(dblprec), intent(in) :: pold(ndim,np)
      complex(dblprec), intent(out) :: pnew(ndim,np)
      logical mfeof
      integer i, j

      call mfrewind(sparsematfile)
      pnew = 0

      ! Multiply vector with diagonal elements of the matrix
      do
        call readmatrixbuffer(sparsematfile, .true., mfeof)
        if (mfeof) then
          exit
        end if

        do i=1, sparsematfile%nwd
          do j=1, np
            ! The matrix is complex symmetric
            pnew(sparsematfile%idx(i), j) = sparsematfile%diag(i) * pold(sparsematfile%idx(i), j)
          end do
        end do
      end do

      ! Multiply vector with off-diagonal elements of the matrix
      do
        call readmatrixbuffer(sparsematfile, .false., mfeof)
        if (mfeof) then
          exit
        end if

        do i=1, sparsematfile%nwo
          do j=1, np
            ! The matrix is complex symmetric where the upper triangle is stored
            pnew(sparsematfile%jox(i), j) = pnew(sparsematfile%jox(i), j)            &
     &                    + sparsematfile%off(i) * pold(sparsematfile%iox(i), j)
            pnew(sparsematfile%iox(i), j) = pnew(sparsematfile%iox(i), j)            &
     &                    + sparsematfile%off(i) * pold(sparsematfile%jox(i), j)
          end do
        end do
      end do
      end subroutine matrixvector


!
! Purely real"ized" version.  All imaginary parts are set to zero.
!

!       subroutine matrixvector(ndim, np, pold, pnew)
!       use sparse
!       implicit none
!
!       integer, intent(in) :: ndim, np
!       complex(dblprec), intent(inout) :: pold(ndim,np)
!       complex(dblprec), intent(out) :: pnew(ndim,np)
!       logical mfeof
!       integer i, j
!       real(dblprec) r
!       complex(dblprec) z
!
!       call mfrewind(sparsematfile)
!       pnew = 0
!       do i=1, ndim
!         do j=1, np
!           z = pold(i, j)
!           r = real(z, dblprec)
!           pold(i, j) = cmplx(r, 0.0D0, dblprec)
!         end do
!       end do
!
!       ! Multiply vector with diagonal elements of the matrix
!       do
!         call readmatrixbuffer(sparsematfile, .true., mfeof)
!         if (mfeof) then
!           exit
!         end if
!
!         do i=1, sparsematfile%nwd
!           do j=1, np
!             ! The matrix is complex symmetric
!             z = sparsematfile%diag(i)
!             r = real(z, dblprec)
!             pnew(sparsematfile%idx(i), j) = cmplx(r, 0.0D0, dblprec) * pold(sparsematfile%idx(i), j)
!           end do
!         end do
!       end do
!
!       ! Multiply vector with off-diagonal elements of the matrix
!       do
!         call readmatrixbuffer(sparsematfile, .false., mfeof)
!         if (mfeof) then
!           exit
!         end if
!
!         do i=1, sparsematfile%nwo
!           do j=1, np
!             ! The matrix is complex symmetric where the upper triangle is stored
!             z = sparsematfile%off(i)
!             r = real(z, dblprec)
!             pnew(sparsematfile%jox(i), j) = pnew(sparsematfile%jox(i), j)            &
!      &                    + cmplx(r, 0.0D0, dblprec) * pold(sparsematfile%iox(i), j)
!             pnew(sparsematfile%iox(i), j) = pnew(sparsematfile%iox(i), j)            &
!      &                    + cmplx(r, 0.0D0, dblprec) * pold(sparsematfile%jox(i), j)
!           end do
!         end do
!       end do
!       end subroutine matrixvector
