!
! fxneval --- evaluate the function f on a grid
! Kevin Christ, Mark Baertschy, Christian Buth
! 24 Aug 2006
!
! Fifth Order Finite Element function evaluation
!
! f is the function, evaluated at the
! points in the vector r, defined by expansion coefficients coef
! in terms of fifth-order polynomial finite elements for
! subintervals defined by nodes.
!
! this is for a radial grid (0 to Rmax)
! half basis functions of types 1 and 2 exist on the first and last intervals
!
!
! requirements:
!    coef = vector of expansion coefficients
!    nodes = data-structure defining finite element nodes
!    r = vector of grid points to evaluate the function
!


      subroutine fxneval1(f,d,r,ngrd,coef,nodes)
      use fortrantl
      use FiniteElements
      implicit none

      integer, intent(in) :: ngrd
      type(FEBnodes), intent(in) :: nodes
      real(dblprec), intent(out) :: f(ngrd), d(ngrd)
      real(dblprec), intent(in) :: r(ngrd), coef(nodes%numbasis)
      integer left, right, nbasis, nodeptr, coptr, j, k
      real(dblprec) h, xp, xq

      f(:) = 0.0D0
      d(:) = 0.0D0

      ! number of half-basis functions for left and right ends
      ! default value is two for both.
      left  = nodes%numleft
      right = nodes%numright

      nbasis = nodes%numbasis
      if (nbasis == 0) return

      nodeptr = 1
      coptr = 0
      do j = 1, ngrd
        if (r(j) >= nodes%nodes(nodeptr)) exit
      end do

      coptr = 1
      do j = j, ngrd
        ! Only fulfilled for r(j) == 0
        if (r(j) == nodes%nodes(nodeptr)) then
          if (left < 3) then
            f(j) = 0.0D0
            d(j) = coef(coptr)
          else
            f(j) = coef(coptr)
            d(j) = coef(coptr+1)
          end if
        else
          exit
        end if
      end do

      nodeptr = nodeptr + 1
      h = nodes%nodes(nodeptr) - nodes%nodes(nodeptr-1)
      coptr = left-2

!       if (nodes%numnds > 2) then
        k = ngrd + 1
        do j=j, ngrd
          if (r(j) >= nodes%nodes(nodeptr)) then
            k = j - 1
            exit
          end if
          xp = (nodes%nodes(nodeptr)-r(j))/h
          xq = (r(j)-nodes%nodes(nodeptr-1))/h
          do k = (3-left), 2
            f(j) = f(j) +           coef(coptr+k)*((-h)**k)*FEbasfun(xp,k)
            d(j) = d(j) - 1.0D0/h * coef(coptr+k)*((-h)**k)*FEbasd1 (xp,k)
          end do
          do k = 0, 2
            f(j) = f(j) +           coef(coptr+3+k)*((h)**k)*FEbasfun(xq,k)
            d(j) = d(j) + 1.0D0/h * coef(coptr+3+k)*((h)**k)*FEbasd1 (xq,k)
          end do
          !! HACK!! FIX THIS!!
          k = j
        end do
        j = k
!       else
!         do j = j, ngrd
!           if (r(j) >= nodes%nodes(nodeptr)) exit
!           xp = (nodes%nodes(nodeptr)-r(j))/h
!           xq = (r(j)-nodes%nodes(nodeptr-1))/h
!           do k = (3-left), 2
!             f(j) = f(j) + coef(coptr+k)*((-h)**k)*FEbasfun(xp,k)
!             d(j) = d(j) - coef(coptr+k)*((-h)**k)*FEbasd1(xp,k)
!           end do
!           do k = (3-right), 2
!             f(j) = f(j) + coef(coptr+right+k)*((h)**k)*FEbasfun(xq,k)
!             d(j) = d(j) + coef(coptr+right+k)*((h)**k)*FEbasd1(xq,k)
!           end do
!         end do
!       end if

      j = j + 1
      do j = j, ngrd
        if (r(j) > nodes%nodes(nodeptr)) then
          nodeptr = nodeptr + 1
          coptr = coptr + 3
          if (nodeptr >= nodes%numnds) then
            !! hack!! fix this!!
            k = j - 1
            exit
          end if
          h = nodes%nodes(nodeptr) - nodes%nodes(nodeptr-1)
        end if
        if (r(j) == nodes%nodes(nodeptr)) then
          f(j) = coef(coptr+3)
          d(j) = coef(coptr+4)
        else
          xp = (nodes%nodes(nodeptr)-r(j))/h
          xq = (r(j)-nodes%nodes(nodeptr-1))/h
          do k = 0, 2
            f(j) = f(j) +           coef(coptr+k)*((-h)**k)*FEbasfun(xp,k)
            d(j) = d(j) - 1.0D0/h * coef(coptr+k)*((-h)**k)*FEbasd1 (xp,k)
          end do
          do k = 0, 2
            f(j) = f(j) +           coef(coptr+3+k)*((h)**k)*FEbasfun(xq,k)
            d(j) = d(j) + 1.0D0/h * coef(coptr+3+k)*((h)**k)*FEbasd1 (xq,k)
          end do
        end if
        !! hack!! fix this!!
        k = j
      end do

      if (nodeptr > nodes%numnds) return

      j = k + 1
      do j = j, ngrd
        if (r(j) > nodes%nodes(nodeptr)) return
        if (r(j) == nodes%nodes(nodeptr)) then
          if (right == 3) then
            f(j) = coef(coptr+3)
            d(j) = coef(coptr+4)
          end if
          if (right == 2) then
            f(j) = 0.0D0
            d(j) = coef(coptr+3)
          end if
        else
          xp = (nodes%nodes(nodeptr)-r(j))/h
          xq = (r(j)-nodes%nodes(nodeptr-1))/h
          do k = 0, 2
            f(j) = f(j) +           coef(coptr+k)*((-h)**k)*FEbasfun(xp,k)
            d(j) = d(j) - 1.0D0/h * coef(coptr+k)*((-h)**k)*FEbasd1 (xp,k)
          end do
          do k = (3-right), 2
            f(j) = f(j) +           coef(coptr+right+k)*((h)**k)*FEbasfun(xq,k)
            d(j) = d(j) + 1.0D0/h * coef(coptr+right+k)*((h)**k)*FEbasd1 (xq,k)
          end do
        endif
      end do
      end subroutine fxneval1


!
! fxneval2 --- evaluate a matrix of eigenvectors with the same
!       dimension as the finite element basis set on a grid
!

!       subroutine fxneval2(f,r,ngrd,coef,nodes)
!       use fortrantl
!       use FiniteElements
!       implicit none
!
!       integer, intent(in) :: ngrd
!       type(FEBnodes), intent(in) :: nodes
!       real(dblprec), intent(in) :: r(ngrd), coef(nodes%numbasis,nodes%numbasis)
!       real(dblprec), intent(out) :: f(ngrd,ngrd)
! !       , d(ngrd,ngrd)
!       integer nbasis, j
!       real(dblprec), allocatable :: temp(:,:), coef1(:)
!
!       f(:,:) = 0.0D0
! !       d(:,:) = 0.0D0
!       nbasis = nodes%numbasis
!       allocate(temp(ngrd,nbasis), coef1(nbasis))
!
!       do j = 1, nbasis
!         call fxneval1(temp(1,j),r,ngrd,coef(1,j),nodes)
!       end do
!
!       do j = 1, ngrd
!         coef1(:) = temp(j,:)
!         call fxneval1(f(1,j),r,ngrd,coef1,nodes)
!       end do
!
!       deallocate(temp, coef1)
!       end subroutine fxneval2
