SUBROUTINE twoelectron( nstate, nl, ngrd, egfxn, xh, CoulInt ) 

  implicit none

  INTEGER :: nstate, nl, ngrd
  REAL(kind=8) :: egfxn(ngrd,nstate,nl)
  REAL(kind=8) :: xh
  REAL(kind=8) :: CoulInt(nstate)
  REAL(kind=8) :: auxa, auxb, trap
  INTEGER :: i, j, l
  REAL(kind=8), ALLOCATABLE :: worka(:), workb(:), workc(:)
  
  ALLOCATE( worka(ngrd), workb(ngrd), workc(ngrd) )

!! Calculate selected two-electron Coulomb matrix elements

!! If the core shells are skipped, then for example in the 
!! case of fluorine, egfxn(i,1,1) refers to the 2s radial
!! wave function, egfxn(i,1,2) refers to 2p, and 
!! egfxn(j,l,3) refers to the lth d orbital calculated.

  do l = 1, nstate
      
     do j = 1, ngrd
        
        do i = 1, j
           auxa = dble(i)/dble(j)
           auxa = auxa**3
           worka(i) = auxa * egfxn(i,1,2) * egfxn(i,1,1)
        end do
        auxa = trap(worka,j,xh)
        
        do i = j, ngrd
           auxb = dble(j)/dble(i)
           auxb = auxb**3
           workb(i-j+1) = auxb * egfxn(i,1,2) * egfxn(i,1,1)
        end do
        auxb = trap(workb,ngrd-j+1,xh)
        
        workc(j) = auxa + auxb
        workc(j) = workc(j) * egfxn(j,1,2) * egfxn(j,l,3)
        
     end do
     
     CoulInt(l) = trap(workc,ngrd,xh)
     CoulInt(l) = 4.d0*CoulInt(l)
     write(25,*) l, CoulInt(l)
     
  end do

  RETURN
END SUBROUTINE twoelectron
